import { BadRequestError } from '@/common/exception/badRequest/BadRequestError';
import { ServerError } from '@/common/exception/ServerError';
import Axios, { AxiosRequestConfig, AxiosResponse, isAxiosError } from 'axios';
import { isObject } from 'lodash';

export async function axiosGetRequest(
  url: string,
  config?: AxiosRequestConfig,
  errorHandler?: (res: AxiosResponse) => Error,
) {
  try {
    return await Axios.get(url, { ...config, timeout: 100000 });
  } catch (e) {
    if (isAxiosError(e)) {
      console.error(e.response);
      if (e.response) {
        if (errorHandler) {
          throw errorHandler(e.response);
        }
        if (isObject(e.response?.data)) {
          const { data: errorData } = e.response;
          throw BadRequestError.default(
            (errorData as any)?.errorMessage ||
              (errorData as any)?.error ||
              (errorData as any)?.error_description,
          );
        }
        throw BadRequestError.default(e.response?.data);
      } else if (e.request) {
        throw BadRequestError.default(
          JSON.stringify(e.response?.data || e.response),
        );
      }
    }
    throw ServerError.default(e.message);
  }
}

export async function axiosPostRequest(
  url: string,
  data?: any,
  config?: AxiosRequestConfig,
  errorHandler?: (res: AxiosResponse) => Error,
) {
  try {
    // await 不能删
    return await Axios.post(url, data, { ...config, timeout: 10000 });
  } catch (e) {
    if (isAxiosError(e)) {
      if (e.response) {
        console.error(e.response, 'request error');
        if (errorHandler) {
          throw errorHandler(e.response);
        }
        if (isObject(e.response?.data)) {
          const { data: errorData } = e.response;
          throw BadRequestError.default(
            (errorData as any)?.errorMessage ||
              (errorData as any)?.error ||
              (errorData as any)?.error_description,
          );
        }
        throw BadRequestError.default(e.response?.data);
      } else if (e.request) {
        throw BadRequestError.default(
          JSON.stringify(e.response?.data || e.response),
        );
      }
    }
    throw ServerError.default(e.message);
  }
}

export async function axiosPatchRequest(
  url: string,
  data?: any,
  config?: AxiosRequestConfig,
  errorHandler?: (res: AxiosResponse) => Error,
) {
  try {
    // await 不能删
    return await Axios.patch(url, data, { ...config, timeout: 10000 });
  } catch (e) {
    if (isAxiosError(e)) {
      if (e.response) {
        console.error(e.response?.data);
        if (errorHandler) {
          throw errorHandler(e.response);
        }
        if (isObject(e.response?.data)) {
          const { data: errorData } = e.response;
          throw BadRequestError.default(
            (errorData as any)?.errorMessage ||
              (errorData as any)?.error ||
              (errorData as any)?.error_description,
          );
        }
        throw BadRequestError.default(e.response?.data);
      } else if (e.request) {
        throw BadRequestError.default(
          JSON.stringify(e.response?.data || e.response),
        );
      }
    }
    throw ServerError.default(e.message);
  }
}

export async function axiosJsonGetRequest(
  url: string,
  sp: Record<string, any>,
  headers?: Record<string, any>,
): Promise<Record<string, any>> {
  const endpoint = new URL(url);
  for (const [k, v] of Object.entries(sp)) {
    endpoint.searchParams.set(k, v);
  }

  const { data } = await axiosGetRequest(endpoint.toString(), {
    headers: {
      ...headers,
      Accept: 'application/json',
    },
  });

  if (typeof data !== 'object') {
    throw BadRequestError.default(
      `API 返回值不是 JSON 格式，实际格式为 ${typeof data}`,
    );
  }

  return data;
}

export async function axiosJsonPostRequest(
  url: string,
  params: Record<string, any>,
  headers?: Record<string, any>,
): Promise<Record<string, any>> {
  const { data } = await axiosPostRequest(url, params, {
    headers: {
      ...headers,
      Accept: 'application/json',
    },
  });

  if (typeof data !== 'object') {
    throw BadRequestError.default(
      `API 返回值不是 JSON 格式，实际格式为 ${typeof data}`,
    );
  }

  return data;
}

export async function axiosJsonPatchRequest(
  url: string,
  params: Record<string, any>,
  headers?: Record<string, any>,
): Promise<Record<string, any>> {
  const { data } = await axiosPatchRequest(url, params, {
    headers: {
      ...headers,
      Accept: 'application/json',
    },
  });

  if (typeof data !== 'object') {
    throw BadRequestError.default(
      `API 返回值不是 JSON 格式，实际格式为 ${typeof data}`,
    );
  }

  return data;
}
